//
//  GLExportCompositer.h
//  GLMapKit
//
//  Created by Jeffrey Early on 2/15/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>

@class GLMapDataSource;
@interface GLExportCompositer : NSObject <GLDrawContext> {
	double _projectedUnitsPerPixel;
	double _scaleFactor;
	GLAffineTransform *_translationTransformation;
	GLAffineTransform *_scaleTransformation;
	NSRect _bounds;
	GLRect _projectedBounds;
	GLMapDataSource  *_mapDataSource;
	GLMapProjection *_mapProjection;
	GLClippingRegion *_clippingRegion;
	BOOL didShiftToPixelBoundaries;
}

/************************************************/
/*		Initialization							*/
/************************************************/

#pragma mark -
#pragma mark Initialization
#pragma mark

// The data source must not be nil -- the bounds are in degrees and the resolution in meters per pixel.
- (GLExportCompositer *) initWithDataSource: (GLMapDataSource *) dataSource bounds: (GLRect) bounds resolution: (double) resolution;

// The data source can be nil, but if one is given, the projection must come from the data source. The bounds and scale are all in projected units.
// If a data source is given where tiles are present, then the bounds will be shifted by up to one pixel in order to draw on pixel boundaries.
- (GLExportCompositer *) initWithDataSource: (GLMapDataSource *) dataSource projection: (GLMapProjection *) projection bounds: (GLRect) bounds resolution: (double) resolution;

// The data source *can* be nil, in which case no background tiles will be requested. The projection must come from the data source.
// The bounds and scale are all in projected units.
- (GLExportCompositer *) initWithProjection: (GLMapProjection *) projection bounds: (GLRect) bounds resolution: (double) resolution;

/************************************************/
/*		Exporter Accessors						*/
/************************************************/

#pragma mark -
#pragma mark Exporter Accessors
#pragma mark

// returns an NSImage of the datasource tiles composited together
- (NSImage *) backgroundImage;

// returns an NSImage of the datasource tiles composited together, restricted to some subset of the compositer's bounds.
- (NSImage *) backgroundImageForRect: (NSRect) bounds;

// returns an NSImage with the spatial objects composited together
- (NSImage *) layerWithSpatialObjects: (NSArray *) spatialObjectArray;

// For internal use to align drawing on appropriate pixel boundaries.
@property BOOL didShiftToPixelBoundaries;
- (void) shiftToPixelBoundaries;

/************************************************/
/*		Geometry Draw Context Protocol			*/
/************************************************/

#pragma mark -
#pragma mark Geometry Draw Context Protocol
#pragma mark

// stores an object representing the projection employed by the draw context, the data source coordinates.
// The geometry will use this object as a weak key to cache a projected version geometry.
@property(strong) GLMapProjection *mapProjection;

// used for scaling the fonts and geometry widths. Scale factor of 1.0 assumes 72 dpi.
@property double scaleFactor;

// The currently displayed resolution of the DrawContext in projected units per pixel.
// Used to remove extraneous points in an optimized path, same as projectedResolution
@property double projectedUnitsPerPixel;

// The current display bounds of the DrawContext, in projected coordinates.
@property GLRect projectedBounds;

// Transform from projected (data source) coordinates to pixel coordinates
@property(readonly, strong) GLAffineTransform *projectedToPixelTransform;

// The is a more detailed version of the above three properties and includes some precomputed properties useful for 
// speedy clipping.
@property(strong) GLClippingRegion *clippingRegion;

// Should return yes if the compositer will be drawing with the same geometry objects multiple times
@property(readonly) BOOL shouldCacheSpatialObjectGeometry;

// Should return yes if the compositer is actively scaling. This *may* cause quick and dirty representations
// of complex geometry to be rendered in order to improve rendering times.
@property(readonly) BOOL inLiveScaling;

/************************************************/
/*		Map Tile DataSource						*/
/************************************************/

#pragma mark -
#pragma mark Map Tile DataSource
#pragma mark

// owner of map tiles and definer of coordinates
@property(strong) GLMapDataSource *mapDataSource;

/************************************************/
/*		Pixel Coordinates						*/
/************************************************/

#pragma mark -
#pragma mark Pixel Coordinates
#pragma mark

// units in pixels
@property(strong) GLAffineTransform *translationTransformation;

// units in pixels/projectedUnit
@property(strong) GLAffineTransform *scaleTransformation;

@property(readonly) GLAffineTransform *pixelToProjectedTransform;

// in pixels
@property NSRect bounds;

// If the world wraps, we might need to draw things in some multiple of the primary coordinates,
// and even multiple times. This function returns the values of n you should loop through and multiply by.
// You still need to check that your bounds are on the screen
- (void) translationMinimum: (int *) nmin maximum: (int *) nmax distance: (int *) pixelWidth;


@end
